unit Random_Generator;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Basic_Form, StdCtrls, ExtCtrls, ComCtrls, Statistics, MtxVec, TeeProcs,
  TeEngine, Chart, Buttons, Series, StatChar;

type
  TfrmRandomGen = class(TfrmBasic)
    Panel2: TPanel;
    Label1: TLabel;
    CBRandomG: TComboBox;
    Label2: TLabel;
    EditNumber: TEdit;
    UpDown1: TUpDown;
    Chart1: TChart;
    SpeedButton1: TSpeedButton;
    Series1: THistogramSeries;
    GroupBox1: TGroupBox;
    lblPar1: TLabel;
    lblPar2: TLabel;
    lblPar3: TLabel;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Label3: TLabel;
    Edit4: TEdit;
    UpDown2: TUpDown;
    CheckBox1: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure CBRandomGChange(Sender: TObject);
    procedure Chart1UndoZoom(Sender: TObject);
  private
    { Private declarations }
    v1: TVec;
    VecCount, VecBins: TVec;
    MsgText : String;
    procedure Estimate;
  public
    { Public declarations }
  end;

var
  frmRandomGen: TfrmRandomGen;

implementation

Uses MtxVecTee, Math387;

{$R *.DFM}

procedure TfrmRandomGen.FormShow(Sender: TObject);
begin
  CBRandomG.ItemIndex := 5; { Gamma distribution }
  CBRandomGChange(Self);
  SpeedButton1Click(Self); { show data }
end;

procedure TfrmRandomGen.FormCreate(Sender: TObject);
begin
  inherited;
  v1 := TVec.Create;
  VecCount := TVec.Create;
  VecBins := TVec.Create;
end;

procedure TfrmRandomGen.FormDestroy(Sender: TObject);
begin
  v1.Destroy;
  VecCount.Destroy;
  VecBins.Destroy;
  inherited;
end;

procedure TfrmRandomGen.SpeedButton1Click(Sender: TObject);
var s1,s2: TSample;
    i1,i2,i3: Integer;
begin
  v1.Size(UpDown1.Position);
  case CBRandomG.ItemIndex of
    0 : begin { Beta }
          s1 := StrToFloat(Edit1.Text);
          s2 := StrToFloat(Edit2.Text);
          RandomBeta(s1,s2,v1);
        end;
    1 : begin { Binomial }
          i1 := StrToInt(Edit1.Text);
          s1 := StrToFloat(Edit2.Text);
          RandomBinom(i1,s1,v1);
        end;
    2 : begin { ChiSquared }
          i1 := StrToInt(Edit1.Text);
          RandomChiSquare(i1,v1);
        end;
    3 : begin { Exponential }
          s1 := StrToFloat(Edit1.Text);
          RandomExponent(s1,v1);
        end;
    4 : begin { F }
          i1 := StrToInt(Edit1.Text);
          i2 := StrToInt(Edit2.Text);
          RandomF(i1,i2,v1);
        end;
    5 : begin { Gamma }
          s1 := StrToFloat(Edit1.Text);
          s2 := StrToFloat(Edit2.Text);
          RandomGamma(s1,s2,v1);
        end;
    6 : begin { Geometric }
          s1 := StrToFloat(Edit1.Text);
          RandomGeometric(s1,v1);
        end;
    7 : begin { Hypergeometric }
          i1 := StrToInt(Edit1.Text);
          i2 := StrToInt(Edit2.Text);
          i3 := StrToInt(Edit3.Text);
          RandomHypGeometric(i1,i2,i3,v1);
        end;
    8 : begin { Log Normal }
          s1 := StrToFloat(Edit1.Text);
          s2 := StrToFloat(Edit2.Text);
          RandomLogNormal(s1,s2,v1);
        end;
    9 : begin { Negative binomial }
          i1 := StrToInt(Edit1.Text);
          s1 := StrToFloat(Edit2.Text);
          RandomNegBinom(i1,s1,v1);
        end;
    10 : begin { Normal }
          s1 := StrToFloat(Edit1.Text);
          s2 := StrToFloat(Edit2.Text);
          RandomNormal(s1,s2,v1);
        end;
    11 : begin { Poisson }
          s1 := StrToFloat(Edit1.Text);
          RandomPoisson(s1,v1);
        end;
    12 : begin { Rayleigh }
          s1 := StrToFloat(Edit1.Text);
          RandomRayleigh(s1,v1);
        end;
    13 : begin { Student }
          i1 := StrToInt(Edit1.Text);
          RandomStudent(i1,v1);
        end;
    14 : begin { continuous uniform }
          s1 := StrToFloat(Edit1.Text);
          s2 := StrToFloat(Edit2.Text);
          RandomUniform(s1,s2,v1);
        end;
    15 : begin { Discrete uniform }
          i1 := StrToInt(Edit1.Text);
          RandomUniformD(i1,v1);
        end;
    16 : begin { Weibull }
          s1 := StrToFloat(Edit1.Text);
          s2 := StrToFloat(Edit2.Text);
          RandomWeibull(s1,s2,v1);
        end;
  end;
  Chart1.SubTitle.Text.Clear;
  { do this only if you want to calculate parameters }
  if CheckBox1.Checked then
  begin
    Estimate;
    Chart1.SubTitle.Text.Add(MsgText);
  end;
  { draw dataset }
  Histogram(v1,UpDown2.Position,VecCount,VecBins,true);
  DrawValues(VecBins,VecCount,Series1,false);
end;

procedure TfrmRandomGen.CBRandomGChange(Sender: TObject);
begin
  Case CBRandomG.ItemIndex of
    0:  begin { Beta }
          lblPar1.Caption := 'a';
          lblPar2.Caption := 'b';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',1.5);
          Edit2.Text := FormatFloat('0.00',0.9);
        end;
    1:  begin { Binomial }
          lblPar1.Caption := 'N';
          lblPar2.Caption := 'p';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := '10';
          Edit2.Text := FormatFloat('0.00',0.4);
        end;
    2:  begin { ChiSquared }
          lblPar1.Caption := 'Nu';
          lblPar2.Visible := false;
          lblPar3.Visible := false;
          Edit1.Text := '4';
        end;
    3:  begin { Exponential }
          lblPar1.Caption := 'Mu';
          lblPar2.Visible := false;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00', 1.8);
        end;
    4:  begin { F }
          lblPar1.Caption := 'Nu1';
          lblPar2.Caption := 'Nu2';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := '3';
          Edit2.Text := '8';
        end;
    5:  begin { Gamma }
          lblPar1.Caption := 'a';
          lblPar2.Caption := 'b';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',2.1);
          Edit2.Text := FormatFloat('0.00',0.3);
        end;
    6:  begin { Geometric }
          lblPar1.Caption := 'P';
          lblPar2.Visible := false;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',0.7);
        end;
    7:  begin { HyperGeometric }
          lblPar1.Caption := 'M';
          lblPar2.Caption := 'K';
          lblPar3.Caption := 'N';
          lblPar2.Visible := true;
          lblPar3.Visible := true;
          Edit1.Text := '50';
          Edit2.Text := '30';
          Edit3.Text := '20';
        end;
    8:  begin { LogNormal }
          lblPar1.Caption := 'mu';
          lblPar2.Caption := 'sigma';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',5.0);
          Edit2.Text := FormatFloat('0.00',0.2);
        end;
    9:  begin { Negative binomial }
          lblPar1.Caption := 'R';
          lblPar2.Caption := 'p';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := '15';
          Edit2.Text := FormatFloat('0.00',0.65);
        end;
    10:  begin { Normal-Gauss }
          lblPar1.Caption := 'mu';
          lblPar2.Caption := 'sigma';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',5.0);
          Edit2.Text := FormatFloat('0.00',0.2);
        end;
    11:  begin { Poisson }
          lblPar1.Caption := 'Lambda';
          lblPar2.Visible := false;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',7.2);
        end;
    12:  begin { Rayleigh }
          lblPar1.Caption := 'b';
          lblPar2.Visible := false;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',0.2);
        end;
    13:  begin { Student }
          lblPar1.Caption := 'Nu';
          lblPar2.Visible := false;
          lblPar3.Visible := false;
          Edit1.Text := '4';
        end;
    14:  begin { Continuous Uniform }
          lblPar1.Caption := 'a';
          lblPar2.Caption := 'b';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',-2.5);
          Edit2.Text := FormatFloat('0.00',3.9);
        end;
    15:  begin { Discrete Uniform }
          lblPar1.Caption := 'N';
          lblPar2.Visible := false;
          lblPar3.Visible := false;
          Edit1.Text := '20';
        end;
    16:  begin { Weibull }
          lblPar1.Caption := 'a';
          lblPar2.Caption := 'b';
          lblPar2.Visible := true;
          lblPar3.Visible := false;
          Edit1.Text := FormatFloat('0.00',1.5);
          Edit2.Text := FormatFloat('0.00',2.3);
        end;
    end;
    Edit2.Visible := lblPar2.Visible;
    Edit3.Visible := lblPar3.Visible;
end;

procedure TfrmRandomGen.Chart1UndoZoom(Sender: TObject);
begin
  Chart1.LeftAxis.AutomaticMaximum := true;
end;

procedure TfrmRandomGen.Estimate;
var Par0,Par1: TSample;
    Int0: Integer;
begin
  Screen.Cursor := crHourGlass;
  Case CBRandomG.ItemIndex of
    0:  begin
          BetaFit(v1,Par0,Par1);
          MsgText := 'a = '+FormatFloat('0.0000',Par0)+' ; '+ 'b = '+FormatFloat('0.0000',Par1);
        end;
    3: begin
          ExponentFit(v1,Par0);
          MsgText := 'mu = '+FormatFloat('0.0000',Par0);
        end;
    5:  begin
          GammaFit(v1,Par0,Par1);
          MsgText := 'a = '+FormatFloat('0.0000',Par0)+' ; '+ 'b = '+FormatFloat('0.0000',Par1);
        end;
    6:  begin
          GeometricFit(v1,Par0);
          MsgText := 'P = '+FormatFloat('0.0000',Par0);
        end;
    10:  begin
          NormalFit(v1,Par0,Par1);
          MsgText := 'mu = '+FormatFloat('0.0000',Par0)+' ; '+ 'sigma = '+FormatFloat('0.0000',Par1);
        end;
    11: begin
          PoissonFit(v1,Par0);
          MsgText := 'lambda = '+FormatFloat('0.0000',Par0);
        end;
    12: begin
          RayleighFit(v1,Par0);
          MsgText := 'b = '+FormatFloat('0.0000',Par0);
        end;
    14: begin
          UniformFit(v1,Par0,Par1);
          MsgText := 'a = '+FormatFloat('0.0000',Par0)+' ; '+ 'b = '+FormatFloat('0.0000',Par1);
        end;
    15: begin
          UniformDFit(v1,Int0);
          MsgText := 'N = '+IntToStr(Int0);
        end;
    16: begin
          WeibullFit(v1,Par0,Par1);
          MsgText := 'a = '+FormatFloat('0.0000',Par0)+' ; '+ 'b = '+FormatFloat('0.0000',Par1);
        end;
    else MsgText := 'Not yet supported';
  end;
  MsgText := 'Estimate (MLE) : ' + MsgText; 
  Screen.Cursor := crDefault;
end;

initialization
  RegisterClass(TfrmRandomGen);

end.
